@@ -18,6 +18,7 @@ use sequoia_openpgp as openpgp;
 use openpgp_card::crypto_data::Cryptogram;
 use openpgp_card::{CardApp, Error};
 
+use crate::sq_util;
 use crate::PublicKey;
 
 pub struct CardDecryptor<'a> {
@@ -39,31 +40,31 @@ impl<'a> CardDecryptor<'a> {
         policy: &dyn Policy,
     ) -> Result<CardDecryptor<'a>, Error> {
         // Get the fingerprint for the decryption key from the card.
-        let ard = ca.get_application_related_data()?;
-        let fps = ard.get_fingerprints()?;
+        let ard = ca.application_related_data()?;
+        let fps = ard.fingerprints()?;
         let fp = fps.decryption();
 
         if let Some(fp) = fp {
             // Transform into Sequoia Fingerprint
             let fp = openpgp::Fingerprint::from_bytes(fp.as_bytes());
 
-            // Find the matching encryption-capable (sub)key in `cert`
-            let keys: Vec<_> = cert
-                .keys()
-                .with_policy(policy, None)
-                .for_storage_encryption()
-                .for_transport_encryption()
-                .filter(|ka| ka.fingerprint() == fp)
-                .map(|ka| ka.key())
-                .collect();
-
-            // Exactly one matching (sub)key should be found. If not, fail!
-            if keys.len() == 1 {
-                let public = keys[0].clone();
-                Ok(Self { ca, public })
+            if let Some(vk) =
+                sq_util::get_subkey_by_fingerprint(cert, policy, &fp, false)?
+            {
+                if vk.for_storage_encryption() || vk.for_transport_encryption()
+                {
+                    let public = vk.key().clone();
+                    Ok(Self { ca, public })
+                } else {
+                    Err(Error::InternalError(anyhow!(
+                        "(Sub)key {} in the cert isn't encryption capable",
+                        fp
+                    )))
+                }
             } else {
                 Err(Error::InternalError(anyhow!(
-                    "Failed to find a matching (sub)key in cert"
+                    "Failed to find (sub)key {} in cert",
+                    fp
                 )))
             }
         } else {
@@ -103,6 +104,12 @@ impl<'a> crypto::Decryptor for CardDecryptor<'a> {
                 mpi::PublicKey::ECDH { ref curve, .. },
             ) => {
                 let dm = if curve == &Curve::Cv25519 {
+                    assert_eq!(
+                        e.value()[0],
+                        0x40,
+                        "Unexpected shape of decrypted Cv25519 data"
+                    );
+
                     // Ephemeral key without header byte 0x40
                     Cryptogram::ECDH(&e.value()[1..])
                 } else {
@@ -119,7 +126,7 @@ impl<'a> crypto::Decryptor for CardDecryptor<'a> {
                 if curve == &Curve::NistP256 && dec.len() == 65 {
                     assert_eq!(
                         dec[0], 0x04,
-                        "unexpected shape of decrypted data"
+                        "Unexpected shape of decrypted NistP256 data"
                     );
 
                     // see Gnuk src/call-ec.c:82
@@ -158,14 +165,13 @@ impl<'a> DecryptionHelper for CardDecryptor<'a> {
         for pkesk in pkesks {
             // Only attempt decryption if the KeyIDs match
             // (this check is an optimization)
-            if pkesk.recipient() == &self.public.keyid() {
-                if pkesk
+            if pkesk.recipient() == &self.public.keyid()
+                && pkesk
                     .decrypt(self, sym_algo)
                     .map(|(algo, session_key)| dec_fn(algo, &session_key))
                     .unwrap_or(false)
-                {
-                    return Ok(Some(self.public.fingerprint()));
-                }
+            {
+                return Ok(Some(self.public.fingerprint()));
             }
         }
 
